#include <bits/stdc++.h>

#pragma GCC optimize "-O3"

#define endl '\n'
#define trace(x) cerr << #x << " = " << endl

using namespace std;

void setIO() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
}

int n;
vector<vector<int>> children;
vector<int> colors;

void read() {
    cin >> n;

    children.resize(n + 1);
    for (int i = 1; i < n; i++) {
        int x, y;
        cin >> x >> y;

        children[x].emplace_back(y);
        children[y].emplace_back(x);
    }

    colors.resize(n + 1);
    for (int i = 1; i <= n; i++) {
        cin >> colors[i];
    }
}

vector<int> par;

void dfsPar(int root) {
    for (int adj : children[root]) {
        if (adj != par[root]) {
            par[adj] = root;
            dfsPar(adj);
        }
    }
}

vector<int> tour, level, first;

void dfsTour(int root, int lvl) {
    tour.emplace_back(root);
    level.emplace_back(lvl);
    if (first[root] == -1) {
        first[root] = tour.size() - 1;
    }

    for (int adj : children[root]) {
        if (adj != par[root]) {
            dfsTour(adj, lvl + 1);

            tour.emplace_back(root);
            level.emplace_back(lvl);
        }
    }
}

int lca(int a, int b) {
    if (first[a] > first[b]) {
        swap(a, b);
    }

    int pos = first[a];
    for (int i = first[a]; i <= first[b]; i++) {
        if (level[i] < level[pos]) {
            pos = i;
        }
    }

    return tour[pos];
}

void colorPath(int u, int v, int c) {
    int l = lca(u, v);

    int node = u;
    while (node != l) {
        colors[node] = c;
        node = par[node];
    }

    node = v;
    while (node != l) {
        colors[node] = c;
        node = par[node];
    }

    colors[l] = c;
}

void colorSubTree(int root, int c) {
    colors[root] = c;

    for (int adj : children[root]) {
        if (adj != par[root]) {
            colorSubTree(adj, c);
        }
    }
}

int query(int u, int v) {
    if (u == v) {
        return 0;
    }

    int l = lca(u, v);

    if (u == l) {
        int prNode = v, node = par[v];
        int answer = 0;

        while (node != l) {
            if (colors[prNode] != colors[node]) {
                answer++;
            }

            prNode = node;
            node = par[node];
        }

        if (colors[u] != colors[prNode]) {
            answer++;
        }

        return answer;
    }

    if (v == l) {
        int prNode = u, node = par[u];
        int answer = 0;

        while (node != l) {
            if (colors[prNode] != colors[node]) {
                answer++;
            }

            prNode = node;
            node = par[node];
        }

        if (colors[v] != colors[prNode]) {
            answer++;
        }

        return answer;
    }

    int prNode = u, node = par[u];
    int answer = 0;

    while (node != l) {
        if (colors[node] != colors[prNode]) {
            answer++;
        }

        prNode = node;
        node = par[node];
    }

    if (colors[l] != colors[prNode]) {
        answer++;
    }

    prNode = v, node = par[v];

    while (node != l) {
        if (colors[node] != colors[prNode]) {
            answer++;
        }

        prNode = node;
        node = par[node];
    }

    if (colors[l] != colors[prNode]) {
        answer++;
    }

    return answer;
}

void solve() {
    par.resize(n + 1, -1);
    dfsPar(1);

    first.resize(n + 1, -1);
    dfsTour(1, 0);

    int q;
    cin >> q;

    while (q--) {
        int type;
        cin >> type;

        if (type == 1) {
            int u, v, c;
            cin >> u >> v >> c;

            colorPath(u, v, c);
        } else if (type == 2) {
            int u, c;
            cin >> u >> c;

            colorSubTree(u, c);
        } else {
            int u, v;
            cin >> u >> v;

            cout << query(u, v) << endl;
        }
    }
}

int main() {
    setIO();

    read();

    solve();
}
