#include <bits/stdc++.h>

#pragma GCC optimize "-O3"

#define endl '\n'
#define trace(x) cerr << #x << " = " << endl

using namespace std;

void setIO() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
}

int n;
vector<vector<int>> children;
vector<int> colors;

void read() {
    cin >> n;

    children.resize(n + 1);
    for (int i = 1; i < n; i++) {
        int x, y;
        cin >> x >> y;

        children[x].emplace_back(y);
        children[y].emplace_back(x);
    }

    colors.resize(n + 1);
    for (int i = 1; i <= n; i++) {
        cin >> colors[i];
    }
}

vector<int> par;

void dfsPar(int root) {
    for (int adj : children[root]) {
        if (adj != par[root]) {
            par[adj] = root;
            dfsPar(adj);
        }
    }
}

vector<int> tour, level, first;

void dfsTour(int root, int lvl) {
    tour.emplace_back(root);
    level.emplace_back(lvl);
    if (first[root] == -1) {
        first[root] = tour.size() - 1;
    }

    for (int adj : children[root]) {
        if (adj != par[root]) {
            dfsTour(adj, lvl + 1);

            tour.emplace_back(root);
            level.emplace_back(lvl);
        }
    }
}

vector<int> lcaTree;

void buildLcaTree(int node, int low, int high) {
    if (low == high) {
        lcaTree[node] = low;
        return;
    }

    int mid = (low + high) >> 1;
    buildLcaTree(node << 1, low, mid);
    buildLcaTree(node << 1 | 1, mid + 1, high);

    if (level[lcaTree[node << 1]] <= level[lcaTree[node << 1 | 1]]) {
        lcaTree[node] = lcaTree[node << 1];
    } else {
        lcaTree[node] = lcaTree[node << 1 | 1];
    }
}

int query(int node, int low, int high, int ql, int qr) {
    if (high < ql || low > qr) {
        return -1;
    }

    if (low >= ql && high <= qr) {
        return lcaTree[node];
    }

    int mid = (low + high) >> 1;
    int p1 = query(node << 1, low, mid, ql, qr);
    int p2 = query(node << 1 | 1, mid + 1, high, ql, qr);

    if (p1 == -1) {
        return p2;
    }

    if (p2 == -1) {
        return p1;
    }

    if (level[p1] <= level[p2]) {
        return p1;
    } else {
        return p2;
    }
}

int lca(int a, int b) {
    if (first[a] > first[b]) {
        swap(a, b);
    }

    return tour[query(1, 0, tour.size() - 1, first[a], first[b])];
}

vector<int> dp;

void dfsDp(int root) {
    if (par[root] != -1) {
        int p = par[root];
        dp[root] = dp[p];
        if (colors[root] != colors[p]) {
            dp[root]++;
        }
    }

    for (int adj : children[root]) {
        if (adj != par[root]) {
            dfsDp(adj);
        }
    }
}

int query(int u, int v) {
    int l = lca(u, v);
    return (dp[u] + dp[v] - 2 * dp[l]);
}

void solve() {
    par.resize(n + 1, -1);
    dfsPar(1);

    first.resize(n + 1, -1);
    dfsTour(1, 0);

    lcaTree.resize(level.size() << 2);
    buildLcaTree(1, 0, level.size() - 1);

    dp.resize(n + 1, 0);
    dfsDp(1);

    int q;
    cin >> q;

    while (q--) {
        int type, u, v;
        cin >> type >> u >> v;

        cout << query(u, v) << endl;
    }
}

int main() {
    setIO();

    read();

    solve();
}

/*
6
1 2
2 6
2 3
3 4
3 5
1 1 3 1 2 1
3
3 6 4
3 1 5
3 3 5
*/
