#include <bits/stdc++.h>

#pragma GCC optimize "-O3"

#define endl '\n'
#define trace(x) cerr << #x << " = " << endl

using namespace std;

void setIO() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
}

int n;
vector<vector<int>> children;
vector<int> colors;

void read() {
    cin >> n;

    children.resize(n + 1);
    for (int i = 1; i < n; i++) {
        int x, y;
        cin >> x >> y;

        children[x].emplace_back(y);
        children[y].emplace_back(x);
    }

    colors.resize(n + 1);
    for (int i = 1; i <= n; i++) {
        cin >> colors[i];
    }
}

vector<int> par;

void dfsPar(int root) {
    for (int adj : children[root]) {
        if (adj != par[root]) {
            par[adj] = root;
            dfsPar(adj);
        }
    }
}

vector<int> in, low;
int dfsTimer = 1;

void dfsInLow(int root) {
    in[root] = low[root] = dfsTimer++;

    for (int adj : children[root]) {
        if (adj != par[root]) {
            dfsInLow(adj);
            low[root] = max(low[root], low[adj]);
        }
    }
}

vector<int> tour, level, first;

void dfsTour(int root, int lvl) {
    tour.emplace_back(in[root]);
    level.emplace_back(lvl);
    if (first[in[root]] == -1) {
        first[in[root]] = tour.size() - 1;
    }

    for (int adj : children[root]) {
        if (adj != par[root]) {
            dfsTour(adj, lvl + 1);

            tour.emplace_back(in[root]);
            level.emplace_back(lvl);
        }
    }
}

vector<int> lcaTree;

void buildLcaTree(int node, int low, int high) {
    if (low == high) {
        lcaTree[node] = low;
        return;
    }

    int mid = (low + high) >> 1;
    buildLcaTree(node << 1, low, mid);
    buildLcaTree(node << 1 | 1, mid + 1, high);

    if (level[lcaTree[node << 1]] <= level[lcaTree[node << 1 | 1]]) {
        lcaTree[node] = lcaTree[node << 1];
    } else {
        lcaTree[node] = lcaTree[node << 1 | 1];
    }
}

int query(int node, int low, int high, int ql, int qr) {
    if (high < ql || low > qr) {
        return -1;
    }

    if (low >= ql && high <= qr) {
        return lcaTree[node];
    }

    int mid = (low + high) >> 1;
    int p1 = query(node << 1, low, mid, ql, qr);
    int p2 = query(node << 1 | 1, mid + 1, high, ql, qr);

    if (p1 == -1) {
        return p2;
    }

    if (p2 == -1) {
        return p1;
    }

    if (level[p1] <= level[p2]) {
        return p1;
    } else {
        return p2;
    }
}

int lca(int a, int b) {
    if (first[a] > first[b]) {
        swap(a, b);
    }

    return tour[query(1, 0, tour.size() - 1, first[a], first[b])];
}

vector<int> dp;

void dfsDp(int root) {
    int id = in[root];

    if (par[root] != -1) {
        int p = in[par[root]];
        dp[id] = dp[p];
        if (colors[root] != colors[par[root]]) {
            dp[id]++;
        }
    }

    for (int adj : children[root]) {
        if (adj != par[root]) {
            dfsDp(adj);
        }
    }
}

vector<int> dpTree, lazyDpTree;

void buildDpTree(int node, int low, int high) {
    if (low == high) {
        dpTree[node] = dp[low];
        return;
    }

    int mid = (low + high) >> 1;
    buildDpTree(node << 1, low, mid);
    buildDpTree(node << 1 | 1, mid + 1, high);

    dpTree[node] = max(dpTree[node << 1], dpTree[node << 1 | 1]);
}

void push(int node, int low, int high) {
    if (lazyDpTree[node]) {
        dpTree[node] = lazyDpTree[node];

        if (low != high) {
            lazyDpTree[node << 1] = lazyDpTree[node];
            lazyDpTree[node << 1 | 1] = lazyDpTree[node];
        }

        lazyDpTree[node] = 0;
    }
}

void updateDpTree(int node, int low, int high, int ql, int qr, int c) {
    push(node, low, high);

    if (high < ql || low > qr) {
        return;
    }

    if (low >= ql && high <= qr) {
        lazyDpTree[node] = c;
        push(node, low, high);
        return;
    }

    int mid = (low + high) >> 1;
    updateDpTree(node << 1, low, mid, ql, qr, c);
    updateDpTree(node << 1 | 1, mid + 1, high, ql, qr, c);

    dpTree[node] = max(dpTree[node << 1], dpTree[node << 1 | 1]);
}

int getDpValue(int pos) {
    int node = 1, l = 1, r = n;
    while (l < r) {
        push(node, l, r);

        int mid = (l + r) >> 1;
        if (pos <= mid) {
            node <<= 1;
            r = mid;
        } else {
            node = node << 1 | 1;
            l = mid + 1;
        }
    }

    push(node, l, r);
    return dpTree[node];
}

vector<int> colorTree, lazyColorTree;

void buildColorTree(int node, int low, int high) {
    if (low == high) {
        colorTree[node] = colors[in[low]];
        return;
    }

    int mid = (low + high) >> 1;
    buildColorTree(node << 1, low, mid);
    buildColorTree(node << 1 | 1, mid + 1, high);

    colorTree[node] = max(colorTree[node << 1], colorTree[node << 1 | 1]);
}

void pushColor(int node, int low, int high) {
    if (lazyColorTree[node]) {
        colorTree[node] = lazyColorTree[node];

        if (low != high) {
            lazyColorTree[node << 1] = lazyColorTree[node];
            lazyColorTree[node << 1 | 1] = lazyColorTree[node];
        }

        lazyColorTree[node] = 0;
    }
}

void updateColorTree(int node, int low, int high, int ql, int qr, int c) {
    pushColor(node, low, high);

    if (high < ql || low > qr) {
        return;
    }

    if (low >= ql && high <= qr) {
        lazyColorTree[node] = c;
        pushColor(node, low, high);
        return;
    }

    int mid = (low + high) >> 1;
    updateColorTree(node << 1, low, mid, ql, qr, c);
    updateColorTree(node << 1 | 1, mid + 1, high, ql, qr, c);

    colorTree[node] = max(colorTree[node << 1], colorTree[node << 1 | 1]);
}

int getColorValue(int pos) {
    int node = 1, l = 1, r = n;
    while (l < r) {
        pushColor(node, l, r);

        int mid = (l + r) >> 1;
        if (pos <= mid) {
            node <<= 1;
            r = mid;
        } else {
            node = node << 1 | 1;
            l = mid + 1;
        }
    }

    pushColor(node, l, r);
    return colorTree[node];
}

int query(int u, int v) {
    u = in[u];
    v = in[v];
    int l = lca(u, v);
    return (getDpValue(u) + getDpValue(v) - 2 * getDpValue(l));
}

void colorSubTree(int u, int c) {
    int idU = in[u];
    if (u != 1) {
        int p = par[u];
        int idP = in[p];

        if (getColorValue(idU) != getColorValue(idP)) {
            updateDpTree(1, 1, n, idU, idU, getDpValue(idU) - 1);
        }

        if (c != getColorValue(idP)) {
            updateDpTree(1, 1, n, idU, idU, getDpValue(idU) + 1);
        }
    }

    updateDpTree(1, 1, n, idU, low[u], getDpValue(idU));
    updateColorTree(1, 1, n, idU, low[u], c);
}

void solve() {
    par.resize(n + 1, -1);
    dfsPar(1);

    in.resize(n + 1);
    low.resize(n + 1);
    dfsInLow(1);

    first.resize(n + 1, -1);
    dfsTour(1, 0);

    lcaTree.resize(level.size() << 2);
    buildLcaTree(1, 0, level.size() - 1);

    dp.resize(n + 1, 0);
    dfsDp(1);

    dpTree.resize(n << 2);
    lazyDpTree.resize(n << 2, 0);
    buildDpTree(1, 1, n);

    colorTree.resize(n << 2);
    lazyColorTree.resize(n << 2, 0);
    buildColorTree(1, 1, n);

    int q;
    cin >> q;

    while (q--) {
        int type, u, v;
        cin >> type;

        if (type == 2) {
            int u, c;
            cin >> u >> c;

            colorSubTree(u, c);
        } else {
            int u, v;
            cin >> u >> v;

            cout << query(u, v) << endl;
        }
    }
}

int main() {
    setIO();

    read();

    solve();
}

/*
6
1 2
2 6
2 3
3 4
3 5
1 1 3 1 2 1
4
3 6 4
3 1 5
2 2 4
3 3 5
*/
