//#include "gcd.h"
#include "gcd.cpp"
#include <iostream>
#include <random>

#define ull unsigned long long

static const size_t NUM_TRIALS = 4e4;

static ull MAX_X;
static ull MAX_AB;
static ull SEED;

static ull x;

static std::mt19937 generator;
static void generateNumber()
{
    std::uniform_int_distribution<ull> distribution(0, MAX_X - 1);
    x = distribution(generator);
}

static ull numQueries;

ull query(ull a, ull b)
{
    if (a < 0 || b <= 0 || a >= MAX_AB || b >= MAX_AB)
    {
//        cerr << a << " " << b << endl;
        std::cout << "Invalid A or B." << std::endl;
        exit(0);
    }

    ++numQueries;

    a += x;
    while (b != 0)
    {
        ull temp = b;
        b = a % b;
        a = temp;
    }

    return a;
}

static void verifyGuess(ull guess)
{
    if (guess != x)
    {
        cerr << "Expected: " << x << endl;
        cerr << "Received: " << guess << endl;
        std::cout << "Wrong guess." << std::endl;
        exit(0);
    }
}

int main()
{
//    std::cin >> MAX_X >> SEED;

    MAX_X = 1000000000;
    SEED = 123456;

    MAX_AB = MAX_X * 2;
    generator.seed(SEED);
    for (size_t i = 0; i < NUM_TRIALS; ++i)
    {
        generateNumber();
        ull guess = play(MAX_X);
        verifyGuess(guess);
//        cerr << "Accepted!" << endl;
//        cin.get();
    }

    double averageQueries = (long double) numQueries / NUM_TRIALS;
    std::cout << "Average queries: " << averageQueries << std::endl;

    return 0;
}
/**
9817.03
1661.94
50.1
5.52225

10^9:
10.57

10.4655
*/
