#include <bits/stdc++.h>
#include <stdint.h>
#include <assert.h>

#define RULES_CAP 128
#define SKIPS_LEN 32
#define INPUT_POLYMER_CAP (100'000+1)
#define MOD 1'000'000'007

struct Rule {
	char alpha;
	char beta;
	char gamma;
	
	uint64_t dimer_count[SKIPS_LEN][RULES_CAP];
};

int64_t rules_len;
Rule rules[RULES_CAP];

int64_t input_polymer_len;
char input_polymer[INPUT_POLYMER_CAP];

int64_t iterations;

uint64_t monomer_sum[26];

int64_t monomers_to_dimer_lookup[26][26];

void find_dimer_count_for_0th_skip(int64_t idx) {
	int64_t l_idx = monomers_to_dimer_lookup[(int64_t)rules[idx].alpha][(int64_t)rules[idx].gamma];
	int64_t r_idx = monomers_to_dimer_lookup[(int64_t)rules[idx].gamma][(int64_t)rules[idx].beta];
	
	if (l_idx != -1) rules[idx].dimer_count[0][l_idx] += 1;
	if (r_idx != -1) rules[idx].dimer_count[0][r_idx] += 1;
}

void find_dimer_count_for_nth_skip(int64_t idx, int64_t skip) {
	for (int64_t mid = 0; mid < rules_len; mid += 1) {
		if (rules[idx].dimer_count[skip-1][mid] == 0) continue;
		for (int64_t dst = 0; dst < rules_len; dst += 1) {
			rules[idx].dimer_count[skip][dst] += rules[idx].dimer_count[skip-1][mid] * rules[mid].dimer_count[skip-1][dst];
			rules[idx].dimer_count[skip][dst] %= MOD;
		}
	}
}

void add_monomer_count_for_n_iterations(int64_t idx, int64_t iterations, uint64_t *monomer_sum) {
	// printf("Information about input dimer %c%c\n", rules[idx].alpha, rules[idx].beta);

	uint64_t dimer_count[SKIPS_LEN + 1][RULES_CAP] = {};
	
	dimer_count[SKIPS_LEN][idx] += 1;
	
	for (int64_t skip = SKIPS_LEN - 1; skip >= 0; skip -= 1) {
		if (!((1 << skip) & iterations)) {
			for (int64_t i = 0; i < rules_len; i += 1) {
				dimer_count[skip][i] = dimer_count[skip+1][i];
			}
			continue;
		}
		for (int64_t mid = 0; mid < rules_len; mid += 1) {
			if (dimer_count[skip+1][mid] == 0) continue;
			for (int64_t dst = 0; dst < rules_len; dst += 1) {
				dimer_count[skip][dst] += dimer_count[skip+1][mid] * rules[mid].dimer_count[skip][dst];
				dimer_count[skip][dst] %= MOD;
			}
		}
	}
	
	int64_t monomer_count[26] = {};
	for (int64_t i = 0; i < rules_len; i += 1) {
		monomer_count[(int64_t)rules[i].alpha] += dimer_count[0][i];
		monomer_count[(int64_t)rules[i].beta]  += dimer_count[0][i];
		
		// if (dimer_count[0][i]) printf("Dimer %c%c: %lld\n", rules[i].alpha, rules[i].beta, dimer_count[0][i]);
	}
	
	// Note that we add only the inside monomers.
	// The 'shell' of the resulting polymer is added in the loop the calls this function.
	monomer_count[(int64_t)rules[idx].alpha] -= 1;
	monomer_count[(int64_t)rules[idx].beta]  -= 1;
	
	for (int64_t i = 0; i < 26; i += 1) {
		assert(monomer_count[i] % 2 == 0);
		monomer_count[i] /= 2;
		
		
		// if (monomer_count[i]) printf("Monomer %c: %lld\n", (char)i + 'A', monomer_count[i]);

		monomer_sum[i] += monomer_count[i];
		monomer_sum[i] %= MOD;
	}
}

int main() {
	memset(monomers_to_dimer_lookup, -1, sizeof(monomers_to_dimer_lookup));
	
	scanf("%s", input_polymer);
	input_polymer_len = strlen(input_polymer);
	for (int64_t i = 0; i < input_polymer_len; i += 1) {
		input_polymer[i] -= 'A';
	}

	scanf("%lld%lld", &iterations, &rules_len);
	
	for (int64_t i = 0; i < rules_len; i += 1) {
		scanf(" %c %c %c", &rules[i].alpha, &rules[i].beta, &rules[i].gamma);
		rules[i].alpha -= 'A';
		rules[i].beta  -= 'A';
		rules[i].gamma -= 'A';
		monomers_to_dimer_lookup[(int64_t)rules[i].alpha][(int64_t)rules[i].beta] = i;
	}
	
	for (int64_t i = 0; i < rules_len; i += 1) {
		find_dimer_count_for_0th_skip(i);
	}
	
	for (int64_t skip = 1; skip < SKIPS_LEN; skip += 1) {
		for (int64_t i = 0; i < rules_len; i += 1) {
			find_dimer_count_for_nth_skip(i, skip);
		}
	}
	
	for (int64_t i = 0; i < input_polymer_len - 1; i += 1) {
		monomer_sum[(int64_t)input_polymer[i]] += 1;
	
		assert(input_polymer[i]   < 26);
		assert(input_polymer[i+1] < 26);
		int64_t rule_idx = monomers_to_dimer_lookup[(int64_t)input_polymer[i]][(int64_t)input_polymer[i+1]];
		if (rule_idx == -1) continue;
		add_monomer_count_for_n_iterations(rule_idx, iterations, monomer_sum);
	}
	monomer_sum[(int64_t)input_polymer[input_polymer_len-1]] += 1;
	
	for (char i = 0; i < 26; i += 1) {
		printf("%c %llu\n", i + 'A', monomer_sum[(int64_t)i] % MOD);
	}
}

/*
NNCB
4 16
CH B
HH N
CB H
NH C
HB C
HC B
HN C
NN C
BH H
NC B
NB B
BN B
BB N
BC B
CC N
CN C

*/