#include <bits/stdc++.h>
#include <stdint.h>
#include <assert.h>
#include <time.h>

#define RULES_CAP 676
#define SKIPS_LEN 32
#define INPUT_POLYMER_CAP (100'000+1)
#define MOD 2'000'000'014

struct Rule {
    char alpha;
    char beta;
    char gamma;
    
	bool can_occur;
	
    int64_t dimer_count[SKIPS_LEN][RULES_CAP];
};

int64_t rules_len;
Rule rules[RULES_CAP];

int64_t input_polymer_len;
char input_polymer[INPUT_POLYMER_CAP];

int64_t iterations;

int64_t monomer_sum[26];

void find_dimer_count_for_0th_skip(int64_t idx) {
    if (rules[idx].gamma == -1) {
        rules[idx].dimer_count[0][idx] += 1;
        return;
    }
    
    int64_t l_idx = rules[idx].alpha * 26 + rules[idx].gamma;
    int64_t r_idx = rules[idx].gamma * 26 + rules[idx].beta;
    
    rules[idx].dimer_count[0][l_idx] += 1;
    rules[idx].dimer_count[0][r_idx] += 1;
}

void find_dimer_count_for_nth_skip(int64_t idx, int64_t skip) {
    for (int64_t mid = 0; mid < RULES_CAP; mid += 1) {
        if (rules[idx].dimer_count[skip-1][mid] == 0) continue;
        if (rules[mid].gamma == -1) {
            rules[idx].dimer_count[skip][mid] += rules[idx].dimer_count[skip-1][mid];
            continue;
        }
        for (int64_t dst = 0; dst < RULES_CAP; dst += 1) {
            rules[idx].dimer_count[skip][dst] += rules[idx].dimer_count[skip-1][mid] * rules[mid].dimer_count[skip-1][dst];
        }
    }
	
	for (int64_t i = 0; i < RULES_CAP; i += 1) {
		rules[idx].dimer_count[skip][i] %= MOD;
	}
}

void add_monomer_count_for_n_iterations(int64_t idx, int64_t iterations, int64_t *monomer_sum) {
    // printf("Information about input dimer %c%c\n", rules[idx].alpha + 'A', rules[idx].beta + 'A');

    int64_t dimer_count[SKIPS_LEN + 1][RULES_CAP] = {};
    
    dimer_count[SKIPS_LEN][idx] += 1;
    
    for (int64_t skip = SKIPS_LEN - 1; skip >= 0; skip -= 1) {
        if (!((1 << skip) & iterations)) {
            for (int64_t i = 0; i < RULES_CAP; i += 1) {
                dimer_count[skip][i] = dimer_count[skip+1][i];
            }
            continue;
        }
        for (int64_t mid = 0; mid < RULES_CAP; mid += 1) {
            if (dimer_count[skip+1][mid] == 0) continue;
            if (rules[mid].gamma == -1) {
                dimer_count[skip][mid] += dimer_count[skip+1][mid];
                continue;
            }
            for (int64_t dst = 0; dst < RULES_CAP; dst += 1) {
                dimer_count[skip][dst] += dimer_count[skip+1][mid] * rules[mid].dimer_count[skip][dst];
                dimer_count[skip][dst] %= MOD;
            }
        }
    }
    
    int64_t monomer_count[26] = {};
    for (int64_t i = 0; i < RULES_CAP; i += 1) {
        monomer_count[(int64_t)rules[i].alpha] += dimer_count[0][i];
        monomer_count[(int64_t)rules[i].beta]  += dimer_count[0][i];
        
        monomer_count[(int64_t)rules[i].alpha] %= MOD;
        monomer_count[(int64_t)rules[i].beta]  %= MOD;
        
        // if (dimer_count[0][i]) printf("Dimer %c%c: %lld\n", rules[i].alpha + 'A', rules[i].beta + 'A', dimer_count[0][i]);
    }
    
    // Note that we add only the inside monomers.
    // The 'shell' of the resulting polymer is added in the loop the calls this function.
    monomer_count[(int64_t)rules[idx].alpha] -= 1;
    monomer_count[(int64_t)rules[idx].beta]  -= 1;
    
    //assert(monomer_count[(int64_t)rules[idx].alpha] % 2 == 0);
    //assert(monomer_count[(int64_t)rules[idx].beta]  % 2 == 0);
    
    for (int64_t i = 0; i < 26; i += 1) {
        assert(monomer_count[i] % 2 == 0);
        monomer_count[i] /= 2;

        // assert(monomer_count[i] >= 0);		
        
        // if (monomer_count[i]) printf("Monomer %c: %lld\n", (char)i + 'A', monomer_count[i]);

        monomer_sum[i] += monomer_count[i];
        monomer_sum[i] %= MOD;
    }
}

int main() {
    for (int64_t i = 0; i < 26; i += 1) {
        for (int64_t j = 0; j < 26; j += 1) {
            rules[i * 26 + j].alpha = i;
            rules[i * 26 + j].beta  = j;
            rules[i * 26 + j].gamma = -1;
        }
    }		
    scanf("%s", input_polymer);
    input_polymer_len = strlen(input_polymer);
    for (int64_t i = 0; i < input_polymer_len; i += 1) {
        input_polymer[i] -= 'A';
    }
	
	for (int64_t i = 0; i < input_polymer_len-1; i += 1) {
        rules[input_polymer[i] * 26 + input_polymer[i+1]].can_occur = true;
    }

    scanf("%lld%lld", &iterations, &rules_len);
    
    for (int64_t i = 0; i < rules_len; i += 1) {
        char alpha, beta, gamma;
        scanf(" %c %c %c", &alpha, &beta, &gamma);
        alpha -= 'A';
        beta  -= 'A';
        gamma -= 'A';
        rules[(int64_t)alpha * 26 + beta].gamma = gamma;
	
        rules[alpha * 26 + beta] .can_occur = true;
        rules[alpha * 26 + gamma].can_occur = true;
        rules[gamma * 26 + beta] .can_occur = true;
    }
    
	clock_t begin;
	
	begin = clock();
    for (int64_t i = 0; i < RULES_CAP; i += 1) {
        find_dimer_count_for_0th_skip(i);
    }
	//printf("%fms\n", (float)(clock() - begin) * 1000.f / CLOCKS_PER_SEC);
    
	begin = clock();
    for (int64_t skip = 1; skip < SKIPS_LEN; skip += 1) {
        for (int64_t i = 0; i < RULES_CAP; i += 1) {
			if (!rules[i].can_occur) continue;
            find_dimer_count_for_nth_skip(i, skip);
        }
    }
	//printf("%fms\n", (float)(clock() - begin) * 1000.f / CLOCKS_PER_SEC);
    
	begin = clock();
    for (int64_t i = 0; i < input_polymer_len - 1; i += 1) {
        monomer_sum[(int64_t)input_polymer[i]] += 1;
    
        int64_t rule_idx = input_polymer[i] * 26 + input_polymer[i+1];
        add_monomer_count_for_n_iterations(rule_idx, iterations, monomer_sum);
    }
    monomer_sum[(int64_t)input_polymer[input_polymer_len-1]] += 1;
	//printf("%fms\n", (float)(clock() - begin) * 1000.f / CLOCKS_PER_SEC);
    
    for (char i = 0; i < 26; i += 1) {
        printf("%c %lld\n", i + 'A', monomer_sum[(int64_t)i] % (MOD / 2));
    }
}
