#include <iostream>
#include <vector>
#include <cstring>
const int maxn = 1e5+10, maxlog = 18;

int n, q;
int color[maxn], parent[maxn], in[maxn], out[maxn], timer;
int diff[maxn];
int tree[4*maxn], treeColor[4*maxn], lazy[4*maxn], lazyColor[4*maxn], depth[maxn];
int inLca[maxn];
int rmq[maxlog][2*maxn], getLog[2*maxn];
std::vector < int > v[maxn], tour, tourLca;

int cmp(int x, int y)
{
    if (depth[x] < depth[y]) return x;
    return y;
}

void buildRmq()
{
    for (int i = 1 ; i < tourLca.size() ; ++i) rmq[0][i] = tourLca[i];
    for (int log = 1 ; (1 << log) <= tourLca.size() ; ++log)
        for (int i = 1 ; i + (1 << log) - 1 < tourLca.size() ; ++i)
            rmq[log][i] = cmp(rmq[log-1][i], rmq[log-1][i + (1 << log-1)]);

    for (int i = 1 ; i < 2*maxn ; ++i)
    {
        getLog[i] = getLog[i-1];
        if ((1 << getLog[i]+1) < i) ++getLog[i];
    }
}

int findLca(int x, int y)
{
    int l = inLca[x];
    int r = inLca[y];
    if (l > r) std::swap(l, r);
    int log = getLog[r-l+1];
    return cmp(rmq[log][l], rmq[log][r - (1 << log) + 1]);
}

void dfsInOut(int x, int p, int currDiff)
{
    depth[x] = depth[p] + 1;
    parent[x] = p;
    tour.push_back(x);
    tourLca.push_back(x);
    inLca[x] = tourLca.size()-1;
    in[x] = tour.size()-1;
    diff[x] = currDiff;
    for (int i : v[x])
    {
        if (i == p) continue;
        dfsInOut(i, x, currDiff + (color[x] != color[i]));
        tourLca.push_back(x);
    }
    out[x] = tour.size()-1;
}

void init(int l, int r, int node)
{
    if (l == r)
    {
        tree[node] = diff[tour[l]];
        treeColor[node] = color[tour[l]];
        return;
    }

    int mid = (l+r)/2;
    init(l, mid, 2*node);
    init(mid+1, r, 2*node+1);
}

void push(int node, int l, int r)
{
    if (lazy[node] == -1) return;
    if (l == r)
    {
        tree[node] = lazy[node];
        treeColor[node] = lazyColor[node];
    } else
    {
        lazy[2*node] = lazy[node];
        lazy[2*node+1] = lazy[node];
        lazyColor[2*node] = lazyColor[node];
        lazyColor[2*node+1] = lazyColor[node];
    }
    lazy[node] = -1;
}

void update(int l, int r, int node, int queryL, int queryR, int queryVal, int queryColor)
{
    push(node, l, r);
    if (queryL <= l && r <= queryR)
    {
        lazy[node] = queryVal;
        lazyColor[node] = queryColor;
        push(node, l, r);
        return;
    }

    int mid = (l+r)/2;
    if (queryL <= mid) update(l, mid, 2*node, queryL, queryR, queryVal, queryColor);
    if (mid+1 <= queryR) update(mid+1, r, 2*node+1, queryL, queryR, queryVal, queryColor);
}

void pointUpdate(int l, int r, int node, int query_ix, int queryVal)
{
    push(node, l, r);
    if (l == r)
    {
        tree[node] += queryVal;
        return;
    }
    
    int mid = (l+r)/2;
    if (query_ix <= mid) return pointUpdate(l, mid, 2*node, query_ix, queryVal);
    return pointUpdate(mid+1, r, 2*node+1, query_ix, queryVal);
}

int query(int l, int r, int node, int query_ix)
{
    push(node, l, r);
    if (l == r) return tree[node];
    
    int mid = (l+r)/2;
    if (query_ix <= mid) return query(l, mid, 2*node, query_ix);
    return query(mid+1, r, 2*node+1, query_ix);
}

int getColor(int l, int r, int node, int query_ix)
{
    push(node, l, r);
    if (l == r) return treeColor[node];
    
    int mid = (l+r)/2;
    if (query_ix <= mid) return getColor(l, mid, 2*node, query_ix);
    return getColor(mid+1, r, 2*node+1, query_ix);
}

void solve()
{
    memset(lazy, -1, sizeof(lazy));
    tour.push_back(0);
    tourLca.push_back(0);
    dfsInOut(1, 0, 0);
    buildRmq();
    init(1, n, 1);

    // std::cout << "tours\n";
    // std::cout << "SegTree:";
    // for (int i : tour) std::cout << ' ' << i;
    // std::cout << '\n';
    // std::cout << "Lca:";
    // for (int i : tourLca) std::cout << ' ' << i;
    // std::cout << '\n';
    
    std::cin >> q;
    int qtype, x, y, c;
    
    for (int i = 1 ; i <= q ; ++i)
    {
        std::cin >> qtype >> x >> y;
    
        if (qtype == 2)
        {
            int colorX = getColor(1, n, 1, in[x]), colorP = getColor(1, n, 1, in[parent[x]]);
            int addVal = (y != colorP) - (colorX != colorP);
            pointUpdate(1, n, 1, in[x], addVal);
            update(1, n, 1, in[x], out[x], query(1, n, 1, in[x]), y);
        }

        if (qtype == 3)
        {
            int lca = findLca(x, y);
            std::cout << query(1, n, 1, in[x]) + query(1, n, 1, in[y]) - 2*query(1, n, 1, in[lca]) << '\n';
        }

    }

}

void read()
{
    int x, y;
    std::cin >> n;
    for (int i = 1 ; i <= n-1 ; ++i)
    {
        std::cin >> x >> y;
        v[x].push_back(y);
        v[y].push_back(x);
    } 
    for (int i = 1 ; i <= n ; ++i) std::cin >> color[i];
}

void fast_io()
{
    std::ios_base :: sync_with_stdio(0);
    std::cin.tie(nullptr);
    std::cout.tie(nullptr);
}

signed main()
{
    fast_io();
    read();
    solve();

    return 0;
}

/*
6
1 2
2 6
2 3
3 4
3 5
1 2 1 1 3 1
7
3 1 5
3 4 5
3 6 5
2 3 1
3 1 5
2 3 2
3 1 5
*/